# Copyright 2023 Zurich Instruments AG
# SPDX-License-Identifier: Apache-2.0

# converter functions for data type package 'scheduled_experiment'
#  AUTOGENERATED, DO NOT EDIT
from typing import Any as AnyDSL

from laboneq.core.types.compiled_experiment import PulseInstance as PulseInstanceDSL
from laboneq.core.types.compiled_experiment import PulseMapEntry as PulseMapEntryDSL
from laboneq.core.types.compiled_experiment import (
    PulseWaveformMap as PulseWaveformMapDSL,
)
from laboneq.core.types.enums.mixer_type import MixerType as MixerTypeDSL
from laboneq.data.scheduled_experiment import MixerType as MixerTypeDATA
from laboneq.data.scheduled_experiment import Optional as OptionalDATA
from laboneq.data.scheduled_experiment import PulseInstance as PulseInstanceDATA
from laboneq.data.scheduled_experiment import PulseMapEntry as PulseMapEntryDATA
from laboneq.data.scheduled_experiment import PulseWaveformMap as PulseWaveformMapDATA
from laboneq.implementation.legacy_adapters.dynamic_converter import convert_dynamic

from .post_process_scheduled_experiment import post_process


def get_converter_function_scheduled_experiment(orig):
    converter_function_directory = {
        PulseInstanceDSL: convert_PulseInstance,
        PulseMapEntryDSL: convert_PulseMapEntry,
        PulseWaveformMapDSL: convert_PulseWaveformMap,
    }
    return converter_function_directory.get(orig)


def convert_MixerType(orig: MixerTypeDSL):
    return (
        next(e for e in MixerTypeDATA if e.name == orig.name)
        if orig is not None
        else None
    )


def convert_PulseInstance(orig: PulseInstanceDSL):
    if orig is None:
        return None
    retval = PulseInstanceDATA()
    retval.amplitude = orig.amplitude
    retval.modulation_phase = orig.modulation_phase
    retval.pulse_pulse_parameters = convert_dynamic(
        orig.pulse_pulse_parameters,
        source_type_hint=AnyDSL,
        target_type_hint=OptionalDATA,
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_scheduled_experiment,
    )
    retval.length = orig.length
    retval.has_marker2 = orig.has_marker2
    retval.channel = orig.channel
    retval.has_marker1 = orig.has_marker1
    retval.needs_conjugate = orig.needs_conjugate
    retval.modulation_frequency = orig.modulation_frequency
    retval.iq_phase = orig.iq_phase
    retval.overlaps = convert_dynamic(
        orig.overlaps,
        source_type_string="List[str]",
        target_type_string="List[str]",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_scheduled_experiment,
    )
    retval.offset_samples = orig.offset_samples
    retval.play_pulse_parameters = convert_dynamic(
        orig.play_pulse_parameters,
        source_type_hint=AnyDSL,
        target_type_hint=OptionalDATA,
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_scheduled_experiment,
    )
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_scheduled_experiment,
    )


def convert_PulseMapEntry(orig: PulseMapEntryDSL):
    if orig is None:
        return None
    retval = PulseMapEntryDATA()
    retval.waveforms = convert_dynamic(
        orig.waveforms,
        source_type_hint=PulseWaveformMapDSL,
        target_type_hint=PulseWaveformMapDATA,
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_scheduled_experiment,
    )
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_scheduled_experiment,
    )


def convert_PulseWaveformMap(orig: PulseWaveformMapDSL):
    if orig is None:
        return None
    retval = PulseWaveformMapDATA()
    retval.sampling_rate = orig.sampling_rate
    retval.length_samples = orig.length_samples
    retval.instances = convert_dynamic(
        orig.instances,
        source_type_hint=PulseInstanceDSL,
        target_type_hint=PulseInstanceDATA,
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_scheduled_experiment,
    )
    retval.signal_type = orig.signal_type
    retval.mixer_type = convert_MixerType(orig.mixer_type)
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_scheduled_experiment,
    )
